@@ -11,7 +11,7 @@
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
 #
-# This program is distributed in the hope that it will be useful, 
+# This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -40,24 +40,28 @@ import uuid
 #
 #-------------------------------------------------------------------------
 from .git_revision import get_git_revision
-from .constfunc import get_env_var, conv_to_unicode
+from .constfunc import get_env_var
+from ..version import VERSION, VERSION_TUPLE, major_version
+from .utils.resourcepath import ResourcePath
+from .utils.grampslocale import GrampsLocale
+
 #-------------------------------------------------------------------------
 #
-# Gramps Version
+# Gramps program name
 #
 #-------------------------------------------------------------------------
-PROGRAM_NAME   = "Gramps"
-from gramps.version import VERSION, VERSION_TUPLE, major_version
+PROGRAM_NAME = "Gramps"
+
 #-------------------------------------------------------------------------
 #
-# Standard GRAMPS Websites
+# Standard Gramps Websites
 #
 #-------------------------------------------------------------------------
-URL_HOMEPAGE    = "http://gramps-project.org/"
+URL_HOMEPAGE = "http://gramps-project.org/"
 URL_MAILINGLIST = "http://sourceforge.net/mail/?group_id=25770"
-URL_BUGHOME     = "http://bugs.gramps-project.org"
-URL_BUGTRACKER  = "http://bugs.gramps-project.org/bug_report_page.php"
-URL_WIKISTRING  = "http://gramps-project.org/wiki/index.php?title="
+URL_BUGHOME = "http://gramps-project.org/bugs"
+URL_BUGTRACKER = "http://gramps-project.org/bugs/bug_report_page.php"
+URL_WIKISTRING = "http://gramps-project.org/wiki/index.php?title="
 URL_MANUAL_PAGE = "Gramps_%s_Wiki_Manual" % major_version
 URL_MANUAL_DATA = '%s_-_Entering_and_editing_data:_detailed' % URL_MANUAL_PAGE
 URL_MANUAL_SECT1 = '%s_-_part_1' % URL_MANUAL_DATA
@@ -65,7 +69,7 @@ URL_MANUAL_SECT2 = '%s_-_part_2' % URL_MANUAL_DATA
 URL_MANUAL_SECT3 = '%s_-_part_3' % URL_MANUAL_DATA
 WIKI_FAQ = "FAQ"
 WIKI_KEYBINDINGS = "Gramps_%s_Wiki_Manual_-_Keybindings" % major_version
-WIKI_EXTRAPLUGINS= "%s_Addons" % major_version
+WIKI_EXTRAPLUGINS = "%s_Addons" % major_version
 WIKI_EXTRAPLUGINS_RAWDATA = "Plugins%s&action=raw" % major_version
 
 #-------------------------------------------------------------------------
@@ -73,13 +77,13 @@ WIKI_EXTRAPLUGINS_RAWDATA = "Plugins%s&action=raw" % major_version
 # Mime Types
 #
 #-------------------------------------------------------------------------
-APP_FAMTREE     = 'x-directory/normal'
-APP_GRAMPS      = "application/x-gramps"
-APP_GRAMPS_XML  = "application/x-gramps-xml"
-APP_GEDCOM      = "application/x-gedcom"
-APP_GRAMPS_PKG  = "application/x-gramps-package"
-APP_GENEWEB     = "application/x-geneweb"
-APP_VCARD       = ["text/x-vcard", "text/x-vcalendar"]
+APP_FAMTREE = 'x-directory/normal'
+APP_GRAMPS = "application/x-gramps"
+APP_GRAMPS_XML = "application/x-gramps-xml"
+APP_GEDCOM = "application/x-gedcom"
+APP_GRAMPS_PKG = "application/x-gramps-package"
+APP_GENEWEB = "application/x-geneweb"
+APP_VCARD = ["text/x-vcard", "text/x-vcalendar"]
 
 #-------------------------------------------------------------------------
 #
@@ -89,47 +93,56 @@ APP_VCARD       = ["text/x-vcard", "text/x-vcalendar"]
 #
 #-------------------------------------------------------------------------
 if 'GRAMPSHOME' in os.environ:
-    USER_HOME = get_env_var('GRAMPSHOME') 
+    USER_HOME = get_env_var('GRAMPSHOME')
     HOME_DIR = os.path.join(USER_HOME, 'gramps')
 elif 'USERPROFILE' in os.environ:
-    USER_HOME = get_env_var('USERPROFILE') 
+    USER_HOME = get_env_var('USERPROFILE')
     if 'APPDATA' in os.environ:
         HOME_DIR = os.path.join(get_env_var('APPDATA'), 'gramps')
     else:
         HOME_DIR = os.path.join(USER_HOME, 'gramps')
 else:
-    USER_HOME = get_env_var('HOME') 
+    USER_HOME = get_env_var('HOME')
     HOME_DIR = os.path.join(USER_HOME, '.gramps')
+ORIG_HOME_DIR = HOME_DIR
+if 'SAFEMODE' in os.environ:
+    if 'USERPROFILE' in os.environ:
+        USER_HOME = get_env_var('USERPROFILE')
+    else:
+        USER_HOME = get_env_var('HOME')
+    HOME_DIR = get_env_var('SAFEMODE')
 
 
-VERSION_DIR    = os.path.join(
+VERSION_DIR = os.path.join(
     HOME_DIR, "gramps%s%s" % (VERSION_TUPLE[0], VERSION_TUPLE[1]))
 
 CUSTOM_FILTERS = os.path.join(VERSION_DIR, "custom_filters.xml")
 REPORT_OPTIONS = os.path.join(HOME_DIR, "report_options.xml")
-TOOL_OPTIONS   = os.path.join(HOME_DIR, "tool_options.xml")
+TOOL_OPTIONS = os.path.join(HOME_DIR, "tool_options.xml")
+PLACE_FORMATS = os.path.join(HOME_DIR, "place_formats.xml")
 
-ENV_DIR        = os.path.join(HOME_DIR, "env")
-TEMP_DIR       = os.path.join(HOME_DIR, "temp")
-THUMB_DIR      = os.path.join(HOME_DIR, "thumb")
-THUMB_NORMAL   = os.path.join(THUMB_DIR, "normal")
-THUMB_LARGE    = os.path.join(THUMB_DIR, "large")
-USER_PLUGINS   = os.path.join(VERSION_DIR, "plugins")
+ENV_DIR = os.path.join(HOME_DIR, "env")
+TEMP_DIR = os.path.join(HOME_DIR, "temp")
+THUMB_DIR = os.path.join(HOME_DIR, "thumb")
+THUMB_NORMAL = os.path.join(THUMB_DIR, "normal")
+THUMB_LARGE = os.path.join(THUMB_DIR, "large")
+USER_PLUGINS = os.path.join(VERSION_DIR, "plugins")
+USER_CSS = os.path.join(HOME_DIR, "css")
 # dirs checked/made for each Gramps session
 USER_DIRLIST = (USER_HOME, HOME_DIR, VERSION_DIR, ENV_DIR, TEMP_DIR, THUMB_DIR,
-                THUMB_NORMAL, THUMB_LARGE, USER_PLUGINS)
+                THUMB_NORMAL, THUMB_LARGE, USER_PLUGINS, USER_CSS)
+
 
 #-------------------------------------------------------------------------
 #
-# Paths to python modules - assumes that the root directory is one level 
+# Paths to python modules - assumes that the root directory is one level
 # above this one, and that the plugins directory is below the root directory.
 #
 #-------------------------------------------------------------------------
-ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(
-    conv_to_unicode(__file__)), os.pardir))
+ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
 
 sys.path.insert(0, ROOT_DIR)
-git_revision = get_git_revision(ROOT_DIR)
+git_revision = get_git_revision(ROOT_DIR).replace('\n', '')
 if sys.platform == 'win32' and git_revision == "":
     git_revision = get_git_revision(os.path.split(ROOT_DIR)[1])
 #VERSION += git_revision
@@ -138,16 +151,15 @@ if sys.platform == 'win32' and git_revision == "":
 #
 # Glade files
 #
-GLADE_DIR      = os.path.join(ROOT_DIR, "gui", "glade")
-GLADE_FILE     = os.path.join(GLADE_DIR, "gramps.glade")
-PERSON_GLADE   = os.path.join(GLADE_DIR, "edit_person.glade")
-PLUGINS_GLADE  = os.path.join(GLADE_DIR, "plugins.glade")
-MERGE_GLADE    = os.path.join(GLADE_DIR, "mergedata.glade")
-RULE_GLADE     = os.path.join(GLADE_DIR, "rule.glade")
+GLADE_DIR = os.path.join(ROOT_DIR, "gui", "glade")
+GLADE_FILE = os.path.join(GLADE_DIR, "gramps.glade")
+PERSON_GLADE = os.path.join(GLADE_DIR, "edit_person.glade")
+PLUGINS_GLADE = os.path.join(GLADE_DIR, "plugins.glade")
+MERGE_GLADE = os.path.join(GLADE_DIR, "mergedata.glade")
+RULE_GLADE = os.path.join(GLADE_DIR, "rule.glade")
 
 
-PLUGINS_DIR        = os.path.join(ROOT_DIR, "plugins")
-WEB_DIR            = os.path.join(ROOT_DIR, 'webapp')
+PLUGINS_DIR = os.path.join(ROOT_DIR, "plugins")
 
 USE_TIPS = False
 
@@ -161,7 +173,6 @@ else:
 # Paths to data files.
 #
 #-------------------------------------------------------------------------
-from gramps.gen.utils.resourcepath import ResourcePath
 _resources = ResourcePath()
 DATA_DIR = _resources.data_dir
 IMAGE_DIR = _resources.image_dir
@@ -174,12 +185,36 @@ LOGO = os.path.join(IMAGE_DIR, "logo.png")
 SPLASH = os.path.join(IMAGE_DIR, "splash.jpg")
 
 LICENSE_FILE = os.path.join(_resources.doc_dir, 'COPYING')
+
+#-------------------------------------------------------------------------
+#
+# Gramps environment variables dictionary
+#
+#-------------------------------------------------------------------------
+ENV = {
+    "USER_HOME": USER_HOME,
+    "HOME_DIR": HOME_DIR,
+    "VERSION": VERSION,
+    "major_version": major_version,
+    "VERSION_DIR": VERSION_DIR,
+    "ENV_DIR": ENV_DIR,
+    "TEMP_DIR": TEMP_DIR,
+    "THUMB_DIR": THUMB_DIR,
+    "THUMB_NORMAL": THUMB_NORMAL,
+    "THUMB_LARGE": THUMB_LARGE,
+    "USER_PLUGINS": USER_PLUGINS,
+    "ROOT_DIR": ROOT_DIR,
+    "GLADE_DIR": GLADE_DIR,
+    "PLUGINS_DIR": PLUGINS_DIR,
+    "DATA_DIR": DATA_DIR,
+    "IMAGE_DIR": IMAGE_DIR,
+}
+
 #-------------------------------------------------------------------------
 #
 # Init Localization
 #
 #-------------------------------------------------------------------------
-from gramps.gen.utils.grampslocale import GrampsLocale
 GRAMPS_LOCALE = GrampsLocale(localedir=_resources.locale_dir)
 _ = GRAMPS_LOCALE.translation.sgettext
 GTK_GETTEXT_DOMAIN = 'gtk30'
@@ -189,28 +224,28 @@ GTK_GETTEXT_DOMAIN = 'gtk30'
 # About box information
 #
 #-------------------------------------------------------------------------
-COPYRIGHT_MSG  = "© 2001-2006 Donald N. Allingham\n" \
-                 "© 2007-2016 The Gramps Developers"
-COMMENTS       = _("Gramps\n (Genealogical Research and Analysis "
-                   "Management Programming System)\n"
-                   "is a personal genealogy program.")
-AUTHORS        = [
+COPYRIGHT_MSG = "© 2001-2006 Donald N. Allingham\n" \
+                "© 2007-2019 The Gramps Developers"
+COMMENTS = _("Gramps\n (Genealogical Research and Analysis "
+             "Management Programming System)\n"
+             "is a personal genealogy program.")
+AUTHORS = [
     "Alexander Roitman",
-    "Benny Malengier", 
+    "Benny Malengier",
     "Brian Matherly",
-    "Donald A. Peterson", 
-    "Donald N. Allingham", 
-    "David Hampton",  
+    "Donald A. Peterson",
+    "Donald N. Allingham",
+    "David Hampton",
     "Martin Hawlisch",
-    "Richard Taylor", 
+    "Richard Taylor",
     "Tim Waugh",
     "John Ralls"
     ]
-    
+
 AUTHORS_FILE = os.path.join(DATA_DIR, "authors.xml")
 
-DOCUMENTERS    = [
-    'Alexander Roitman', 
+DOCUMENTERS = [
+    'Alexander Roitman',
     ]
 
 #-------------------------------------------------------------------------
@@ -218,13 +253,15 @@ DOCUMENTERS    = [
 # Constants
 #
 #-------------------------------------------------------------------------
-THUMBSCALE       = 96.0
+THUMBSCALE = 96.0
 THUMBSCALE_LARGE = 180.0
-XMLFILE          = "data.gramps"
-NO_SURNAME       = "(%s)" % _("none")
-NO_GIVEN         = "(%s)" % _("none")
-ARABIC_COMMA     = "،"
+XMLFILE = "data.gramps"
+NO_SURNAME = "(%s)" % _("surname|none")
+NO_GIVEN = "(%s)" % _("given-name|none")
+ARABIC_COMMA = "،"
 ARABIC_SEMICOLON = "؛"
+DOCGEN_OPTIONS = 'Docgen Options'
+COLON = _(':') # translators: needed for French, ignore otherwise
 
 #-------------------------------------------------------------------------
 #
@@ -232,68 +269,109 @@ ARABIC_SEMICOLON = "؛"
 #
 #-------------------------------------------------------------------------
 
-# Note: Make sure to edit argparser.py _help string too!
-# (longName, shortName, type , default, flags, descrip , argDescrip)
-POPT_TABLE = [
-    ("config",  'c',  str, None, 0, "Set config setting(s) and start Gramps",  ""),
-    ("open",    'O', str, None, 0, "Open family tree",  "FAMILY_TREE"), 
-    ("create",  'C', str, None, 0, "Create or Open family tree",  "FAMILY_TREE"), 
-    ("import",  'i', str, None, 0, "Import file",       "FILENAME"), 
-    ("export",  'e', str, None, 0, "Export file",       "FILENAME"),
-    ("format",  'f', str, None, 0, 'Specify format',    "FORMAT"), 
-    ("action",  'a', str, None, 0, 'Specify action',    "ACTION"), 
-    ("options", 'p', str, None, 0, 'Specify options',   "OPTIONS_STRING"), 
-    ("debug",   'd', str, None, 0, 'Enable debug logs', "LOGGER_NAME"), 
-    ("",        'l', None, None, 0, 'List Family Trees', ""),
-    ("",        'L', None, None, 0, 'List Family Tree Details', ""),
-    ("show",    's', None, None, 0, "Show config settings",  ""),
-    ("force-unlock", 'u', None, None, 0, 'Force unlock of family tree', ""),
-    ("version", 'v', None, None, 0, 'Show versions', ""),
-]
-
 LONGOPTS = [
-    "action=", 
+    "action=",
     "class=",
     "config=",
     "debug=",
+    "default=",
     "display=",
-    "disable-sound", 
-    "disable-crash-dialog", 
+    "disable-sound",
+    "disable-crash-dialog",
     "enable-sound",
     "espeaker=",
     "export=",
     "force-unlock",
     "format=",
-    "gdk-debug=", 
-    "gdk-no-debug=", 
-    "gtk-debug=", 
-    "gtk-no-debug=", 
-    "gtk-module=", 
+    "gdk-debug=",
+    "gdk-no-debug=",
+    "gtk-debug=",
+    "gtk-no-debug=",
+    "gtk-module=",
     "g-fatal-warnings",
     "help",
-    "import=", 
+    "import=",
     "load-modules=",
-    "list" 
+    "list"
     "name=",
-    "oaf-activate-iid=", 
-    "oaf-ior-fd=", 
+    "oaf-activate-iid=",
+    "oaf-ior-fd=",
     "oaf-private",
     "open=",
+    "username=",
+    "password=",
     "create=",
     "options=",
+    "safe",
     "screen=",
-    "show", 
-    "sm-client-id=", 
-    "sm-config-prefix=", 
+    "show",
+    "sm-client-id=",
+    "sm-config-prefix=",
     "sm-disable",
     "sync",
-    "usage", 
+    "remove=",
+    "usage",
     "version",
-    "qml",
     "yes",
     "quiet",
 ]
 
-SHORTOPTS = "O:C:i:e:f:a:p:d:c:lLthuv?syq"
+SHORTOPTS = "O:U:P:C:i:e:f:a:p:d:c:r:lLthuv?syqSD:"
 
-GRAMPS_UUID =  uuid.UUID('516cd010-5a41-470f-99f8-eb22f1098ad6')
+GRAMPS_UUID = uuid.UUID('516cd010-5a41-470f-99f8-eb22f1098ad6')
+
+#-------------------------------------------------------------------------
+#
+# Fanchart Constants
+#
+#-------------------------------------------------------------------------
+
+PIXELS_PER_GENERATION = 50 # size of radius for generation
+BORDER_EDGE_WIDTH = 10     # empty white box size at edge to indicate parents
+CHILDRING_WIDTH = 12       # width of the children ring inside the person
+TRANSLATE_PX = 10          # size of the central circle, used to move the chart
+PAD_PX = 4                 # padding with edges
+PAD_TEXT = 2               # padding for text in boxes
+
+BACKGROUND_SCHEME1 = 0
+BACKGROUND_SCHEME2 = 1
+BACKGROUND_GENDER = 2
+BACKGROUND_WHITE = 3
+BACKGROUND_GRAD_GEN = 4
+BACKGROUND_GRAD_AGE = 5
+BACKGROUND_SINGLE_COLOR = 6
+BACKGROUND_GRAD_PERIOD = 7
+GENCOLOR = {
+    BACKGROUND_SCHEME1: ((255, 63, 0),
+                         (255, 175, 15),
+                         (255, 223, 87),
+                         (255, 255, 111),
+                         (159, 255, 159),
+                         (111, 215, 255),
+                         (79, 151, 255),
+                         (231, 23, 255),
+                         (231, 23, 121),
+                         (210, 170, 124),
+                         (189, 153, 112)),
+    BACKGROUND_SCHEME2: ((229, 191, 252),
+                         (191, 191, 252),
+                         (191, 222, 252),
+                         (183, 219, 197),
+                         (206, 246, 209)),
+    BACKGROUND_WHITE: ((255, 255, 255),
+                       (255, 255, 255),),
+    }
+
+MAX_AGE = 100
+GRADIENTSCALE = 5
+
+FORM_CIRCLE = 0
+FORM_HALFCIRCLE = 1
+FORM_QUADRANT = 2
+
+COLLAPSED = 0
+NORMAL = 1
+EXPANDED = 2
+
+TYPE_BOX_NORMAL = 0
+TYPE_BOX_FAMILY = 1
