@@ -26,6 +26,8 @@
 from abc import ABCMeta, abstractmethod
 import os
 import shutil
+import re
+from subprocess import Popen, PIPE
 from io import StringIO
 import tempfile
 import logging
@@ -97,9 +99,25 @@ _NOTESIZE = [{'name': _("Tiny"), 'value': "tiny"},
 
 if win():
     _LATEX_FOUND = search_for("lualatex.exe")
+    DETACHED_PROCESS = 8
 else:
     _LATEX_FOUND = search_for("lualatex")
 
+def escape(text):
+    lookup = {
+        '&': '\\&',
+        '%': '\\%',
+        '$': '\\$',
+        '#': '\\#',
+        '_': '\\_',
+        '{': '\\{',
+        '}': '\\}',
+        '~': '\\~{}',
+        '^': '\\^{}',
+        '\\': '\\textbackslash{}'
+        }
+    pattern = re.compile('|'.join([re.escape(key) for key in lookup.keys()]))
+    return pattern.sub(lambda match: lookup[match.group(0)], text)
 
 #------------------------------------------------------------------------------
 #
@@ -324,9 +342,10 @@ class TreeDocBase(BaseDoc, TreeDoc):
         self.write(0, '\\begin{document}\n')
 
         if self.nodecolor == 'preferences':
-            male_bg = config.get('preferences.color-gender-male-death')[1:]
-            female_bg = config.get('preferences.color-gender-female-death')[1:]
-            neuter_bg = config.get('preferences.color-gender-unknown-death')[1:]
+            scheme = config.get('colors.scheme')
+            male_bg = config.get('colors.male-dead')[scheme][1:]
+            female_bg = config.get('colors.female-dead')[scheme][1:]
+            neuter_bg = config.get('colors.unknown-dead')[scheme][1:]
             self.write(0, '\\definecolor{male-bg}{HTML}{%s}\n' % male_bg)
             self.write(0, '\\definecolor{female-bg}{HTML}{%s}\n' % female_bg)
             self.write(0, '\\definecolor{neuter-bg}{HTML}{%s}\n' % neuter_bg)
@@ -390,10 +409,7 @@ class TreeDocBase(BaseDoc, TreeDoc):
         options = ['id=%s' % family.gramps_id]
         if option_list:
             options.extend(option_list)
-        if subgraph_type == 'sandclock':
-            self.write(level, 'sandclock{\n')
-        else:
-            self.write(level, '%s[%s]{\n' % (subgraph_type, ','.join(options)))
+        self.write(level, '%s[%s]{\n' % (subgraph_type, ','.join(options)))
 
     def end_subgraph(self, level):
         self.write(level, '}\n')
@@ -414,8 +430,8 @@ class TreeDocBase(BaseDoc, TreeDoc):
         nick = name.get_nick_name()
         surn = name.get_surname()
         name_parts = [self.format_given_names(name),
-                      '\\nick{{{}}}'.format(nick) if nick else '',
-                      '\\surn{{{}}}'.format(surn) if surn else '']
+                      '\\nick{{{}}}'.format(escape(nick)) if nick else '',
+                      '\\surn{{{}}}'.format(escape(surn)) if surn else '']
         self.write(level+1, 'name = {{{}}},\n'.format(
             ' '.join([e for e in name_parts if e])))
         for eventref in person.get_event_ref_list():
@@ -430,15 +446,22 @@ class TreeDocBase(BaseDoc, TreeDoc):
                         event = db.get_event_from_handle(eventref.ref)
                         self.write_event(db, level+1, event)
         for attr in person.get_attribute_list():
-            if str(attr.get_type()) == 'Occupation':
-                self.write(level+1, 'profession = {%s},\n' % attr.get_value())
-            if str(attr.get_type()) == 'Comment':
-                self.write(level+1, 'comment = {%s},\n' % attr.get_value())
+            # Comparison with 'Occupation' for backwards compatibility with Gramps 5.0
+            attr_type = str(attr.get_type())
+            if attr_type in ('Occupation', _('Occupation')):
+                self.write(level+1, 'profession = {%s},\n' %
+                           escape(attr.get_value()))
+            if attr_type == 'Comment':
+                self.write(level+1, 'comment = {%s},\n' %
+                           escape(attr.get_value()))
         for mediaref in person.get_media_list():
-            media = db.get_object_from_handle(mediaref.ref)
+            media = db.get_media_from_handle(mediaref.ref)
             path = media_path_full(db, media.get_path())
             if os.path.isfile(path):
-                self.write(level+1, 'image = {%s},\n' % path)
+                if win():
+                    path = path.replace('\\', '/')
+                self.write(level+1, 'image = {{%s}%s},\n' %
+                           os.path.splitext(path))
                 break # first image only
         self.write(level, '}\n')
 
@@ -495,7 +518,7 @@ class TreeDocBase(BaseDoc, TreeDoc):
             stop_date = self.format_iso(date.get_stop_ymd(), calendar)
             date_str = date_str + '/' + stop_date
 
-        place = _pd.display_event(db, event)
+        place = escape(_pd.display_event(db, event))
 
         if modifier:
             event_type += '+'
@@ -518,14 +541,14 @@ class TreeDocBase(BaseDoc, TreeDoc):
             if call in first:
                 where = first.index(call)
                 return '{before}\\pref{{{call}}}{after}'.format(
-                    before=first[:where],
-                    call=call,
-                    after=first[where+len(call):])
+                    before=escape(first[:where]),
+                    call=escape(call),
+                    after=escape(first[where+len(call):]))
             else:
                 # ignore erroneous call name
-                return first
+                return escape(first)
         else:
-            return first
+            return escape(first)
 
     def format_iso(self, date_tuple, calendar):
         """
@@ -581,7 +604,7 @@ class TreeTexDoc(TreeDocBase):
         if self._filename[-4:] != ".tex":
             self._filename += ".tex"
 
-        with open(self._filename, "w") as texfile:
+        with open(self._filename, 'w', encoding='utf-8') as texfile:
             texfile.write(self._tex.getvalue())
 
 
@@ -604,11 +627,16 @@ class TreePdfDoc(TreeDocBase):
             self._filename += ".pdf"
 
         with tempfile.TemporaryDirectory() as tmpdir:
-            with open(os.path.join(tmpdir, 'temp.tex'), "w") as texfile:
-                texfile.write(self._tex.getvalue())
-            os.system('lualatex -output-directory %s temp.tex >/dev/null'
-                      % tmpdir)
-            shutil.copy(os.path.join(tmpdir, 'temp.pdf'), self._filename)
+            basename = os.path.basename(self._filename)
+            args = ['lualatex', '-output-directory', tmpdir,
+                    '-jobname', basename[:-4]]
+            if win():
+                proc = Popen(args, stdin=PIPE, stdout=PIPE, stderr=PIPE,
+                             creationflags=DETACHED_PROCESS)
+            else:
+                proc = Popen(args, stdin=PIPE, stdout=PIPE, stderr=PIPE)
+            proc.communicate(input=self._tex.getvalue().encode('utf-8'))
+            shutil.copy(os.path.join(tmpdir, basename), self._filename)
 
 
 #------------------------------------------------------------------------------
