@@ -31,6 +31,7 @@ GrampletView interface.
 from gi.repository import Gdk
 from gi.repository import Gtk
 from gi.repository import Pango
+from xml.sax.saxutils import escape
 import time
 import os
 import configparser
@@ -49,7 +50,7 @@ from gramps.gen.const import URL_MANUAL_PAGE, VERSION_DIR, COLON
 from ..editors import EditPerson, EditFamily
 from ..managedwindow import ManagedWindow
 from ..utils import is_right_click, match_primary_mask, get_link_color
-from .menuitem import add_menuitem
+from ..uimanager import ActionGroup, valid_action_name
 from ..plug import make_gui_option
 from ..plug.quick import run_quick_report_by_name
 from ..display import display_help, display_url
@@ -189,6 +190,13 @@ def logical_true(value):
     """
     return value in ["True", True, 1, "1"]
 
+def make_callback(func, arg):
+    """
+    Generates a callback function based off the passed arguments
+    """
+    return lambda x, y: func(arg)
+
+
 class LinkTag(Gtk.TextTag):
     """
     Class for keeping track of link data.
@@ -637,7 +645,7 @@ class GuiGramplet:
                     person = self.dbstate.db.get_person_from_handle(handle)
                     if person is not None:
                         if event.button == 1: # left mouse
-                            if event.type == Gdk.EventType._2BUTTON_PRESS: # double
+                            if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
                                 try:
                                     EditPerson(self.dbstate,
                                                self.uistate,
@@ -645,7 +653,7 @@ class GuiGramplet:
                                     return True # handled event
                                 except WindowActiveError:
                                     pass
-                            elif event.type == Gdk.EventType.BUTTON_PRESS: # single
+                            elif event.type == Gdk.EventType.BUTTON_PRESS:
                                 self.uistate.set_active(handle, 'Person')
                                 return True # handled event
                         elif is_right_click(event):
@@ -659,7 +667,7 @@ class GuiGramplet:
                                 pass
                 elif link_type == 'Surname':
                     if event.button == 1: # left mouse
-                        if event.type == Gdk.EventType._2BUTTON_PRESS: # double
+                        if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
                             run_quick_report_by_name(self.dbstate,
                                                      self.uistate,
                                                      'samesurnames',
@@ -667,7 +675,7 @@ class GuiGramplet:
                     return True
                 elif link_type == 'Given':
                     if event.button == 1: # left mouse
-                        if event.type == Gdk.EventType._2BUTTON_PRESS: # double
+                        if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
                             run_quick_report_by_name(self.dbstate,
                                                      self.uistate,
                                                      'samegivens_misc',
@@ -675,7 +683,7 @@ class GuiGramplet:
                     return True
                 elif link_type == 'Filter':
                     if event.button == 1: # left mouse
-                        if event.type == Gdk.EventType._2BUTTON_PRESS: # double
+                        if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
                             run_quick_report_by_name(self.dbstate,
                                                      self.uistate,
                                                      'filterbyname',
@@ -700,7 +708,7 @@ class GuiGramplet:
                     family = self.dbstate.db.get_family_from_handle(handle)
                     if family is not None:
                         if event.button == 1: # left mouse
-                            if event.type == Gdk.EventType._2BUTTON_PRESS: # double
+                            if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
                                 try:
                                     EditFamily(self.dbstate,
                                                self.uistate,
@@ -708,7 +716,7 @@ class GuiGramplet:
                                     return True # handled event
                                 except WindowActiveError:
                                     pass
-                            elif event.type == Gdk.EventType.BUTTON_PRESS: # single
+                            elif event.type == Gdk.EventType.BUTTON_PRESS:
                                 self.uistate.set_active(handle, 'Family')
                                 return True # handle event
                         elif is_right_click(event):
@@ -722,7 +730,7 @@ class GuiGramplet:
                                 pass
                 elif link_type == 'PersonList':
                     if event.button == 1: # left mouse
-                        if event.type == Gdk.EventType._2BUTTON_PRESS: # double
+                        if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
                             run_quick_report_by_name(self.dbstate,
                                                      self.uistate,
                                                      'filterbyname',
@@ -731,7 +739,7 @@ class GuiGramplet:
                     return True
                 elif link_type == 'Attribute':
                     if event.button == 1: # left mouse
-                        if event.type == Gdk.EventType._2BUTTON_PRESS: # double
+                        if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
                             run_quick_report_by_name(self.dbstate,
                                                      self.uistate,
                                                      'attribute_match',
@@ -752,8 +760,9 @@ class GridGramplet(GuiGramplet):
     Class that handles the plugin interfaces for the GrampletView.
     """
     TARGET_TYPE_FRAME = 80
-    LOCAL_DRAG_TYPE   = 'GRAMPLET'
-    LOCAL_DRAG_TARGET = (Gdk.atom_intern(LOCAL_DRAG_TYPE, False), 0, TARGET_TYPE_FRAME)
+    LOCAL_DRAG_TYPE = 'GRAMPLET'
+    LOCAL_DRAG_TARGET = Gtk.TargetEntry.new(LOCAL_DRAG_TYPE, 0,
+                                            TARGET_TYPE_FRAME)
 
     def __init__(self, pane, dbstate, uistate, title, **kwargs):
         """
@@ -799,12 +808,8 @@ class GridGramplet(GuiGramplet):
         # source:
         drag = self.gvproperties
         drag.drag_source_set(Gdk.ModifierType.BUTTON1_MASK,
-                             [],
+                             [GridGramplet.LOCAL_DRAG_TARGET],
                              Gdk.DragAction.COPY)
-        tglist = Gtk.TargetList.new([])
-        tg = GridGramplet.LOCAL_DRAG_TARGET
-        tglist.add(tg[0], tg[1], tg[2])
-        drag.drag_source_set_target_list(tglist)
 
         # default tooltip
         msg = _("Drag Properties Button to move and click it for setup")
@@ -967,8 +972,11 @@ class GridGramplet(GuiGramplet):
 
     def set_title(self, new_title, set_override=True):
         # can't do it if already titled that way
-        if self.title == new_title: return True
-        if new_title in self.pane.gramplet_map: return False
+        if self.title == new_title:
+            return True
+        if(new_title in self.pane.gramplet_map or
+           new_title != escape(new_title)):  # avoid XML specific characters
+            return False
         if set_override:
             self.title_override = True
         del self.pane.gramplet_map[self.title]
@@ -1001,6 +1009,8 @@ class GrampletPane(Gtk.ScrolledWindow):
         self.pageview = pageview
         self.pane = self
         self._popup_xy = None
+        self.at_popup_action = None
+        self.at_popup_menu = None
         user_gramplets = self.load_gramplets()
         # build the GUI:
         msg = _("Right click to add gramplets")
@@ -1016,12 +1026,8 @@ class GrampletPane(Gtk.ScrolledWindow):
         self.drag_dest_set(Gtk.DestDefaults.MOTION |
                             Gtk.DestDefaults.HIGHLIGHT |
                             Gtk.DestDefaults.DROP,
-                            [],
+                            [GridGramplet.LOCAL_DRAG_TARGET],
                             Gdk.DragAction.COPY)
-        tglist = Gtk.TargetList.new([])
-        tg = GridGramplet.LOCAL_DRAG_TARGET
-        tglist.add(tg[0], tg[1], tg[2])
-        self.drag_dest_set_target_list(tglist)
         self.connect('drag_drop', self.drop_widget)
         self.eventb.connect('button-press-event', self._button_press)
 
@@ -1350,8 +1356,7 @@ class GrampletPane(Gtk.ScrolledWindow):
         self.place_gramplets(recolumn=True)
         self.show()
 
-    def restore_gramplet(self, obj):
-        name = obj.get_child().get_label()
+    def restore_gramplet(self, name):
         ############### First kind: from current session
         for gramplet in self.closed_gramplets:
             if gramplet.title == name:
@@ -1393,8 +1398,7 @@ class GrampletPane(Gtk.ScrolledWindow):
             else:
                 self.drop_widget(self, gramplet, 0, 0, 0)
 
-    def add_gramplet(self, obj):
-        tname = obj.get_child().get_label()
+    def add_gramplet(self, tname):
         all_opts = get_gramplet_options_by_tname(tname)
         name = all_opts["name"]
         if all_opts is None:
@@ -1438,39 +1442,73 @@ class GrampletPane(Gtk.ScrolledWindow):
             LOG.warning("Can't make gramplet of type '%s'.", name)
 
     def _button_press(self, obj, event):
+        ui_def = (
+            '''    <menu id="Popup">
+        <submenu>
+          <attribute name="action">win.AddGramplet</attribute>
+          <attribute name="label" translatable="yes">Add a gramplet</attribute>
+          %s
+        </submenu>
+        <submenu>
+          <attribute name="action">win.RestoreGramplet</attribute>
+          <attribute name="label" translatable="yes">'''
+            '''Restore a gramplet</attribute>
+          %s
+        </submenu>
+        </menu>
+        ''')
+        menuitem = ('<item>\n'
+                    '<attribute name="action">win.%s</attribute>\n'
+                    '<attribute name="label">%s</attribute>\n'
+                    '</item>\n')
+
         if is_right_click(event):
             self._popup_xy = (event.x, event.y)
             uiman = self.uistate.uimanager
-            ag_menu = uiman.get_widget('/GrampletPopup/AddGramplet')
-            if ag_menu:
-                qr_menu = ag_menu.get_submenu()
-                qr_menu = Gtk.Menu()
-                names = [gplug.name for gplug in PLUGMAN.get_reg_gramplets()
-                         if gplug.navtypes == []
-                            or 'Dashboard' in gplug.navtypes]
-                names.sort()
+            actions = []
+            r_menuitems = ''
+            a_menuitems = ''
+            plugs = [gplug for gplug in PLUGMAN.get_reg_gramplets() if
+                     gplug.navtypes == [] or 'Dashboard' in gplug.navtypes]
+            plugs.sort(key=lambda x: x.name)
+            for plug in plugs:
+                action_name = valid_action_name(plug.id)
+                a_menuitems += menuitem % (action_name, plug.name)
+                actions.append((action_name,
+                                make_callback(self.add_gramplet, plug.name)))
+            names = [gramplet.title for gramplet in self.closed_gramplets]
+            names.extend(opts["title"] for opts in self.closed_opts)
+            names.sort()
+            if len(names) > 0:
                 for name in names:
-                    add_menuitem(qr_menu, name, None,
-                                           self.add_gramplet)
-                ag_menu.set_submenu(qr_menu)
-            rg_menu = uiman.get_widget('/GrampletPopup/RestoreGramplet')
-            if rg_menu:
-                qr_menu = rg_menu.get_submenu()
-                if qr_menu is not None:
-                    rg_menu.set_submenu(None)
-                names = [gramplet.title for gramplet in self.closed_gramplets]
-                names.extend(opts["title"] for opts in self.closed_opts)
-                names.sort()
-                if len(names) > 0:
-                    qr_menu = Gtk.Menu()
-                    for name in names:
-                        add_menuitem(qr_menu, name, None,
-                                               self.restore_gramplet)
-                    rg_menu.set_submenu(qr_menu)
-            self.menu = uiman.get_widget('/GrampletPopup')
-            if self.menu:
-                #GTK3 does not show the popup, workaround: menu as attribute
-                self.menu.popup(None, None, None, None, event.button, event.time)
+                    # 'name' could be non-ASCII when in non-English language
+                    # action names must be in ASCII, so use 'id' instead.
+                    action_name = valid_action_name(str(id(name)))
+                    r_menuitems += menuitem % (action_name, name)
+                    actions.append((action_name,
+                                    make_callback(self.restore_gramplet,
+                                                  name)))
+
+            if self.at_popup_action:
+                uiman.remove_ui(self.at_popup_menu)
+                uiman.remove_action_group(self.at_popup_action)
+            self.at_popup_action = ActionGroup('AtPopupActions',
+                                               actions)
+            uiman.insert_action_group(self.at_popup_action)
+            self.at_popup_menu = uiman.add_ui_from_string([
+                ui_def % (a_menuitems, r_menuitems)])
+            uiman.update_menu()
+
+            menu = uiman.get_widget('Popup')
+            popup_menu = Gtk.Menu.new_from_model(menu)
+            popup_menu.attach_to_widget(obj, None)
+            popup_menu.show_all()
+            if Gtk.MINOR_VERSION < 22:
+                # ToDo The following is reported to work poorly with Wayland
+                popup_menu.popup(None, None, None, None,
+                                 event.button, event.time)
+            else:
+                popup_menu.popup_at_pointer(event)
                 return True
         return False
 
