@@ -54,12 +54,12 @@ from gramps.gen.datehandler import displayer, get_date
 from gramps.gen.display.name import displayer as _nd
 from gramps.gen.display.place import displayer as _pd
 from gramps.gen.utils.place import conv_lat_lon
-from gramps.gui.views.navigationview import NavigationView
 from gramps.gui.views.bookmarks import PersonBookmarks
 from gramps.plugins.lib.maps import constants
 from gramps.plugins.lib.maps.geography import GeoGraphyView
 from gramps.gui.selectors import SelectorFactory
 from gramps.gen.utils.db import (get_birth_or_fallback, get_death_or_fallback)
+from gramps.gui.uimanager import ActionGroup
 
 #-------------------------------------------------------------------------
 #
@@ -67,43 +67,120 @@ from gramps.gen.utils.db import (get_birth_or_fallback, get_death_or_fallback)
 #
 #-------------------------------------------------------------------------
 
-_UI_DEF = '''\
-<ui>
-<menubar name="MenuBar">
-<menu action="GoMenu">
-  <placeholder name="CommonGo">
-    <menuitem action="Back"/>
-    <menuitem action="Forward"/>
-    <separator/>
-    <menuitem action="HomePerson"/>
-    <separator/>
-  </placeholder>
-</menu>
-<menu action="EditMenu">
-  <placeholder name="CommonEdit">
-    <menuitem action="PrintView"/>
-  </placeholder>
-</menu>
-<menu action="BookMenu">
-  <placeholder name="AddEditBook">
-    <menuitem action="AddBook"/>
-    <menuitem action="EditBook"/>
-  </placeholder>
-</menu>
-</menubar>
-<toolbar name="ToolBar">
-<placeholder name="CommonNavigation">
-  <toolitem action="Back"/>
-  <toolitem action="Forward"/>
-  <toolitem action="HomePerson"/>
-  <toolitem action="RefPerson"/>
-</placeholder>
-<placeholder name="CommonEdit">
-  <toolitem action="PrintView"/>
-</placeholder>
-</toolbar>
-</ui>
-'''
+_UI_DEF = [
+    '''
+      <placeholder id="CommonGo">
+      <section>
+        <item>
+          <attribute name="action">win.Back</attribute>
+          <attribute name="label" translatable="yes">_Back</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Forward</attribute>
+          <attribute name="label" translatable="yes">_Forward</attribute>
+        </item>
+      </section>
+      <section>
+        <item>
+          <attribute name="action">win.HomePerson</attribute>
+          <attribute name="label" translatable="yes">_Home</attribute>
+        </item>
+      </section>
+      </placeholder>
+    ''',
+    '''
+      <section id='CommonEdit' groups='RW'>
+        <item>
+          <attribute name="action">win.PrintView</attribute>
+          <attribute name="label" translatable="yes">Print...</attribute>
+        </item>
+      </section>
+    ''',
+    '''
+      <section id="AddEditBook">
+        <item>
+          <attribute name="action">win.AddBook</attribute>
+          <attribute name="label" translatable="yes">_Add Bookmark</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.EditBook</attribute>
+          <attribute name="label" translatable="no">%s...</attribute>
+        </item>
+      </section>
+    ''' % _('Organize Bookmarks'),  # Following are the Toolbar items
+    '''
+    <placeholder id='CommonNavigation'>
+    <child groups='RO'>
+      <object class="GtkToolButton">
+        <property name="icon-name">go-previous</property>
+        <property name="action-name">win.Back</property>
+        <property name="tooltip_text" translatable="yes">'''
+    '''Go to the previous object in the history</property>
+        <property name="label" translatable="yes">_Back</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    <child groups='RO'>
+      <object class="GtkToolButton">
+        <property name="icon-name">go-next</property>
+        <property name="action-name">win.Forward</property>
+        <property name="tooltip_text" translatable="yes">'''
+    '''Go to the next object in the history</property>
+        <property name="label" translatable="yes">_Forward</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    <child groups='RO'>
+      <object class="GtkToolButton">
+        <property name="icon-name">go-home</property>
+        <property name="action-name">win.HomePerson</property>
+        <property name="tooltip_text" translatable="yes">'''
+    '''Go to the default person</property>
+        <property name="label" translatable="yes">_Home</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    <child groups='RO'>
+      <object class="GtkToolButton">
+        <property name="icon-name">gramps-person</property>
+        <property name="action-name">win.RefPerson</property>
+        <property name="tooltip_text" translatable="yes">'''
+    '''Select the person which is the reference for life ways</property>
+        <property name="label" translatable="yes">reference _Person</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    </placeholder>
+    ''',
+    '''
+    <placeholder id='BarCommonEdit'>
+    <child groups='RO'>
+      <object class="GtkToolButton">
+        <property name="icon-name">document-print</property>
+        <property name="action-name">win.PrintView</property>
+        <property name="tooltip_text" translatable="yes">'''
+    '''Print or save the Map</property>
+        <property name="label" translatable="yes">Print...</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    </placeholder>
+    ''']
 
 # pylint: disable=no-member
 # pylint: disable=unused-argument
@@ -219,6 +296,7 @@ class GeoClose(GeoGraphyView):
         self.remove_all_markers()
         self.lifeway_layer.clear_ways()
         self.message_layer.clear_messages()
+        self.message_layer.set_font_attributes(None, None, None)
         active = self.get_active()
         if active:
             indiv1 = self.dbstate.db.get_person_from_handle(active)
@@ -291,18 +369,10 @@ class GeoClose(GeoGraphyView):
         """
         Define action for the reference person button.
         """
-        NavigationView.define_actions(self)
+        GeoGraphyView.define_actions(self)
+        self._add_action('RefPerson', self.select_person)
 
-        self.define_print_actions()
-        self.ref_person = Gtk.ActionGroup(name=self.title + '/Selection')
-        self.ref_person.add_actions([
-            ('RefPerson', 'gramps-person', _('reference _Person'), None,
-            _("Select the person which is the reference for life ways"),
-            self.select_person),
-            ])
-        self._add_action_group(self.ref_person)
-
-    def select_person(self, obj):
+    def select_person(self, *obj):
         """
         Open a selection box to choose the ref person.
         """
